#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jun 29 08:00:40 2017

@author: lessig
"""

import numpy as np
import matplotlib.pyplot as plt


def feEval( h, k, x) :
    
    k = k+1
    xd = np.abs(h*k - x)
    
    return np.maximum( np.zeros(x.size), (h - xd) / h)
    

def feReconstruct( h, coeffs, x) :
    
    vals = np.zeros( x.size)
    for k in range( coeffs.size) :
        vals += coeffs[k] * feEval( h, k, x)
        
    return vals
        
        
def feStiffness( h) :
    
    n = 1/h-1
    
    L = (2/h) * np.eye(n) 
    
    offd = (-1/h) * np.ones(n-1)
    L += np.diag( offd, 1) + np.diag( offd, -1)
  
    return L

    
def feMass( h) :
    
    n = 1/h-1
    
    M = h * np.eye(n) 
    
    offd = (1/4*h) * np.ones(n-1)
    M += np.diag( offd, 1) + np.diag( offd, -1)
    
    return M

    
h = 1/100

L = feStiffness( h)
M = feMass( h)

# constant rhs
# g = np.ones( 1/h - 1)
# linear rhs
g = 0.25 * np.linspace(h,1-h,1/h-1)
gM = np.dot( M, g)

f = np.linalg.solve( L, gM)

xs = np.linspace( 0.0, 1.0, 1000)
ys = feReconstruct( h, f, xs)
plt.plot( xs, ys)